--[[-------------------------------------------------------------------------
Copyright (c) 2010 Jan Erik Hanssen

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
---------------------------------------------------------------------------]]


--[[-------------------------------------------------------------------------
	Configuration
---------------------------------------------------------------------------]]

local defaultConfig = {
	-- geometry
	xpos = 0,
	ypos = -30,

	-- anchor
	anchor = "CENTER",
	attach = "UIParent",

	-- colors
	energyColor = { r = 1, g = 0.86, b = 0.1, opacity = 1 },
	runicColor = { r = 0.1, g = 0.86, b = 1, opacity = 1 },
	rageColor = { r = 0.9, g = 0.1, b = 0.1, opacity = 1 },
	manaColor = { r = 0.1, g = 0.1, b = 0.9, opacity = 1 },
	focusColor = { r = 0.85, g = 0.36, b = 0.25, opacity = 1 },

	-- text
	font = "Fonts\\FRIZQT__.TTF",
	fontSize = "13",
	fontStyle = "OUTLINE",
	
	-- Hide power types
	hideEnergy = "0",
	hideRunic = "0",
	hideRage = "0",
	hideMana = "0",
	hideFocus = "0",

	-- hide when out of combat
	hideOOC = "0",

	-- hide when in the idle state (i.e. max energy or no rage)
	hideIdle = "1",
}

--[[-------------------------------------------------------------------------------
	Code
---------------------------------------------------------------------------------]]

local cfg = {}
for k, v in pairs(defaultConfig) do
	cfg[k] = v
end

if EnergizeDB == nil then
	EnergizeDB = {}
	for k, v in pairs(defaultConfig) do
		EnergizeDB[k] = v
	end
else
	for k, v in pairs(defaultConfig) do
		if EnergizeDB[k] == nil then
			EnergizeDB[k] = v
		end
	end
end

local addon = CreateFrame("Frame", "Energize", UIParent)

-- the allmighty
function addon:new()
	self.energyText = nil
	
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("UNIT_MAXPOWER")
	self:RegisterEvent("UNIT_DISPLAYPOWER")
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")
	self:RegisterEvent("UNIT_EXITED_VEHICLE")

	self:SetScript("OnEvent", self.event)

	self:ClearAllPoints()
	self:SetWidth(100)
	self:SetHeight(50)
	self:SetPoint(cfg.anchor, cfg.attach, cfg.anchor, cfg.xpos, cfg.ypos)
	self:SetMovable(true)
end

local cur = 0
local max = 0
local idle = 0
local next = 0
local type = nil
local incombat = false
local curunit = "player"
local invalidtype = false
local visible = true
local showTypes = {}
local colorTypes = {}

addon.locked = true

function addon:init()
	self.energyText = self:CreateFontString(nil, "OVERLAY")
	self.energyText:SetFont(cfg.font, cfg.fontSize, cfg.fontStyle)
	self.energyText:SetShadowOffset(1, -1)
	self.energyText:SetShadowColor(0, 0, 0, 1)
	self.energyText:SetJustifyH("CENTER")
	self.energyText:ClearAllPoints()
	self.energyText:SetAllPoints(self)
	
	local color = colorTypes[type]
	self.energyText:SetTextColor(color.r, color.g, color.b, color.opacity)
	
	if cfg.hideOOC == "1" then
		visible = false
		self:SetAlpha(0)

		self:RegisterEvent("PLAYER_REGEN_DISABLED")
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	end

	if cfg.hideIdle == "1" and visible then
		local energy = UnitPower(curunit, type)
		if energy == idle then
			visible = false
			self:SetAlpha(0)
		end
	end
end

function addon:updateIdle()
	if type == SPELL_POWER_RAGE or type == SPELL_POWER_RUNIC_POWER then
		idle = 0
	else
		idle = max
	end
end

function addon:updateEnergy()
	if not self.locked then return end
	if not showTypes[type] then
		if visible then
			self:SetAlpha(0)
			visible = false
			invalidtype = true
		end
		return
	end
	
	if invalidtype then
		self:SetAlpha(1)
		visible = true
		invalidtype = false
	end

	if cfg.hideOOC == "1" then
		if visible and not incombat then
			UIFrameFadeOut(self, 0.2)
			visible = false
		elseif not visible and incombat then
			if cfg.hideIdle == "1" and cur == idle then return end

			UIFrameFadeIn(self, 0.2)
			self.fadeInfo.finishedFunc = function() if not visible then self:SetAlpha(0) end end
			visible = true
		end
	end

	if cfg.hideIdle == "1" then
		if cur == idle and visible then
			UIFrameFadeOut(self, 0.2)
			visible = false
		elseif cur ~= idle and not visible then
			if cfg.hideOOC == "1" and not incombat then return end

			UIFrameFadeIn(self, 0.2)
			self.fadeInfo.finishedFunc = function() if not visible then self:SetAlpha(0) end end
			visible = true
		end
	elseif not visible then
		if cfg.hideOOC == "1" and not incombat then return end

		UIFrameFadeIn(self, 0.2)
		self.fadeInfo.finishedFunc = function() if not visible then self:SetAlpha(0) end end
		visible = true
	end
	
	self.energyText:SetText(cur)
end

function addon:onupdate()
	if not self.locked then return end
	if not showTypes[type] then return end

	next = UnitPower(curunit, type)

	if cfg.hideIdle == "1" and not visible and next ~= idle then
		if cfg.hideOOC == "0" or (cfg.hideOOC == "1" and incombat) then
			UIFrameFadeIn(self, 0.2)
			self.fadeInfo.finishedFunc = function() if not visible then self:SetAlpha(0) end end
			visible = true
		end
	end

	if next ~= cur then
		cur = next
		if visible then
			self:updateEnergy()
		end
	end
end

function addon:applySettings()
	for k, v in pairs(EnergizeDB) do
		cfg[k] = v
	end
end

function addon:updateSettings()
	self:applySettings()

	showTypes[SPELL_POWER_MANA] = cfg.hideMana == "0"
	showTypes[SPELL_POWER_RAGE] = cfg.hideRage == "0"
	showTypes[SPELL_POWER_ENERGY] = cfg.hideEnergy == "0"
	showTypes[SPELL_POWER_RUNIC_POWER] = cfg.hideRunic == "0"
	showTypes[SPELL_POWER_FOCUS] = cfg.hideFocus == "0"
	
	colorTypes[SPELL_POWER_MANA] = cfg.manaColor
	colorTypes[SPELL_POWER_RAGE] = cfg.rageColor
	colorTypes[SPELL_POWER_ENERGY] = cfg.energyColor
	colorTypes[SPELL_POWER_RUNIC_POWER] = cfg.runicColor
	colorTypes[SPELL_POWER_FOCUS] = cfg.focusColor

	if self.energyText ~= nil then
		self.energyText:SetFont(cfg.font, cfg.fontSize, cfg.fontStyle)
		local type, _ = UnitPowerType(curunit)
		local color = colorTypes[type]
		self.energyText:SetTextColor(color.r, color.g, color.b, color.opacity)
	end
	
	self:updateEnergy()
end

function addon:toggleLock()
	if self.locked then
		self.locked = false
		self:EnableMouse(true)
		self:RegisterForDrag("LeftButton")
		self:SetScript("OnDragStart", function(self) 
			self:StartMoving()
		end)
		self:SetScript("OnDragStop", function(self)
			self:StopMovingOrSizing()
		end)
		UIFrameFadeIn(self, 0.2)
		visible = true
	else
		self.locked = true
		self:EnableMouse(false)
		self:RegisterForDrag(nil)
		self:SetScript("OnDragStart", nil)
		self:SetScript("OnDragStop", nil)
		self:updateEnergy()
	end
	if self.callbacks ~= nil then
		self.callbacks:Fire("Energize_Locked")
	end
	DEFAULT_CHAT_FRAME:AddMessage(format("Energize frame is now %s.", self.locked and "locked" or "unlocked"))
end

function addon:event(event, ...)
	if event == "UNIT_MAXPOWER" then
		local unit, type = select(1, ...)
		if unit == curunit then
			max = UnitPowerMax(unit, type)
			self:updateIdle()
		end

	elseif event == "VARIABLES_LOADED" then
		self:updateSettings()

	elseif event == "PLAYER_LOGIN" then
		if UnitHasVehicleUI("player") then
			local vehicletype, _ = UnitPowerType("vehicle")
			if showTypes[vehicletype] then
				curunit = "vehicle"
			end
		end

		type, _ = UnitPowerType(curunit)
		max = UnitPowerMax(curunit, type)
		self:updateIdle()
		
		if self.energyText == nil then
			self:SetScript("OnUpdate", self.onupdate)
			self:init()

			SlashCmdList["ENERGIZE"] = function(msg)
				if msg == "reset" then
					EnergizeDB = {}
					for k, v in pairs(defaultConfig) do
						EnergizeDB[k] = v
					end
					addon:updateSettings()
					self:SetUserPlaced(false)
					self:ClearAllPoints()
					self:SetWidth(100)
					self:SetHeight(50)
					self:SetPoint(cfg.anchor, cfg.attach, cfg.anchor, cfg.xpos, cfg.ypos)
					DEFAULT_CHAT_FRAME:AddMessage("All Energize settings reset.")
				elseif msg == "lock" then
					addon:toggleLock()
				else
					DEFAULT_CHAT_FRAME:AddMessage("energize <reset> <lock>")
				end
			end

			SLASH_ENERGIZE1 = "/energize"
		end
	
		self:updateEnergy()

	elseif event == "PLAYER_REGEN_DISABLED" then
		incombat = true
		self:updateEnergy()

	elseif event == "PLAYER_REGEN_ENABLED" then
		incombat = false
		self:updateEnergy()
		
	elseif event == "UNIT_DISPLAYPOWER" then
		local unit = select(1, ...)
		if unit ~= curunit then return end

		local oldtype = type
		type, _ = UnitPowerType(curunit)
		max = UnitPowerMax(curunit, type)
		self:updateIdle()
		if type ~= oldtype then
			local color = colorTypes[type]
			self.energyText:SetTextColor(color.r, color.g, color.b, color.opacity)
		end
		self:updateEnergy()

	elseif event == "UNIT_ENTERED_VEHICLE" then
		local unit = select(1, ...)
		if unit ~= "player" then return end
		
		local vehicletype, _ = UnitPowerType("vehicle")
		if showTypes[vehicletype] then
			curunit = "vehicle"
			local oldtype = type
			type = vehicletype
			max = UnitPowerMax(curunit, type)
			self:updateIdle()
			
			if type ~= oldtype then
				local color = colorTypes[type]
				self.energyText:SetTextColor(color.r, color.g, color.b, color.opacity)
			end
			
			self:updateEnergy()
		end

	elseif event == "UNIT_EXITED_VEHICLE" then
		local unit = select(1, ...)
		if unit ~= "player" then return end

		curunit = "player"
		local oldtype = type
		type, _ = UnitPowerType(curunit)
		if type ~= oldtype then
			local color = colorTypes[type]
			self.energyText:SetTextColor(color.r, color.g, color.b, color.opacity)
		end
		max = UnitPowerMax(curunit, type)
		self:updateIdle()
		
		self:updateEnergy()
	end
end

function addon:initCallbacks()
	if not LibStub then return end
	self.callback = self.callback or LibStub("CallbackHandler-1.0")
	if self.callback ~= nil then
		self.callbacks = self.callbacks or self.callback:New(self)
	end
end

-- and... go!
addon:new()

EnergizeAddon = addon
